﻿using System;
using System.Collections.Generic;
using System.Linq;
using Uccelerate.Helpers;
using Uccelerate.Services.Model;
using UCommerce.Api;
using UCommerce.EntitiesV2;
using UCommerce.Runtime;
using Umbraco.Core.Models;
using Umbraco.Web;

namespace Uccelerate.Public { 
    public class Helpers
    {
        public static ProductVariation BuildProductModel(Product product, string culture, string baseUrl, bool ExtendedModel, IPublishedContent site)
        {

            PriceGroup priceGroup = SiteContext.Current.CatalogContext.CurrentPriceGroup;
            var prices = CatalogLibrary.CalculatePrice(new List<Guid>() { product.Guid }, new List<Guid>() { priceGroup.Guid });

            if (!prices.Items.Any() && product.IsVariant)
            {
                prices = CatalogLibrary.CalculatePrice(new List<Guid>() { product.ParentProduct.Guid }, new List<Guid>() { priceGroup.Guid });
            }

            if (prices.Items.Any())
            {
                var properties = new Dictionary<string, string>();
                var variants = new List<ProductVariation>();
                var price = prices.Items.First(); // only need a single price
                var listPrice = price.ListPriceInclTax.FormatCurrency(priceGroup.Currency.ISOCode); //format price
                var yourPrice = price.PriceInclTax.FormatCurrency(priceGroup.Currency.ISOCode); // format price
                
                if (product.Variants.Count > 0)
                {
                    foreach (var variant in product.Variants)
                    {
                        variants.Add(BuildProductModel(variant, culture, baseUrl, ExtendedModel, site));
                    }
                }

                //add properties
                foreach (var property in product.ProductProperties)
                {
                    if (property.ProductDefinitionField.DisplayOnSite && properties.ContainsKey(property.ProductDefinitionField.Name))
                    {
                        properties.Add(property.ProductDefinitionField.Name, property.GetValue().ToString());
                    }
                }

                //add multilingual properties
                foreach (var property in product.GetProperties(culture).ToList())
                {
                    var definition = property.GetDefinitionField();
                    var propertyVal = property.GetValue().ToString();

                    properties.Add(definition.Name, propertyVal);

                }

                var CanOrder = (product.IsVariant) ? product.ParentProduct.AllowOrdering : product.AllowOrdering;

                var productResponse = new ProductVariation
                {
                    Sku = product.Sku,
                    VariantSku = product.VariantSku,
                    Name = product.GetDescription(culture).DisplayName,
                    Url = ProductHelper.GetUrl(baseUrl, culture, product, site),
                    ListPrice = listPrice,
                    YourPrice = yourPrice,
                    PrimaryImageUrl = ProductHelper.GetPrimaryImage(product),
                    SecondaryImageUrl = ProductHelper.GetSecondaryImage(product),
                    CanOrder = CanOrder,
                    HasVariants = product.Variants.Count > 1,
                    Properties = properties,
                    Variants = variants
                };

                if (ExtendedModel) { 
                    productResponse.ShortDescription = product.GetDescription(culture).ShortDescription;
                    productResponse.LongDescription = product.GetDescription(culture).LongDescription;
                }

                return productResponse;

            }
            return null;
        }
    }
}