﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Threading;
using UCommerce;
using UCommerce.Api;
using UCommerce.EntitiesV2;
using UCommerce.Runtime;
using Uccelerate.Services.Model;
using Umbraco.Web.WebApi;
using System.Web.Http;
using Umbraco.Web;
using System.Web.SessionState;

namespace UccelerateApi
{

    public class SetPaymentAndShipping
    {
        public int PaymentId { get; set; }
        public int ShipmentId { get; set; }
        public int CountryId { get; set; }
    }
    public class SetPaymentAndShippingResponse
    {
        public SetPaymentAndShippingResponse()
        {
        }

        public SetPaymentAndShippingResponse(decimal orderTotal, ICollection<UCommerce.EntitiesV2.Discount> discounts)
        {

            Discounts = new List<Uccelerate.Services.Model.Discount>();

            var currency = SiteContext.Current.CatalogContext.CurrentPriceGroup.Currency;
            OrderTotal = new Money(orderTotal, currency).ToString();
            var priceGroup = SiteContext.Current.CatalogContext.CurrentPriceGroup;

            foreach (var discount in discounts)
            {
                var Tax = new Money(discount.AmountOffTotal*priceGroup.VATRate, currency);
                var price = new Money(discount.AmountOffTotal+Tax.Value, currency).ToString();

                Discounts.Add(new Uccelerate.Services.Model.Discount
                {
                    Name = discount.CampaignItemName,
                    Price = price
                });
            }
        }
        public string OrderTotal { get; set; }
        public ICollection<Uccelerate.Services.Model.Discount> Discounts { get; set; }
    }
    public class CheckoutController : UmbracoApiController, IRequiresSessionState
    {
        [HttpPost]
        public SetPaymentAndShippingResponse SetPaymentAndShipping(SetPaymentAndShipping request)
        {
            //IMPORTANT TO SET FOR PAYPAL
            System.Net.ServicePointManager.SecurityProtocol = System.Net.SecurityProtocolType.Tls12;

            TransactionLibrary.EditBillingInformation(
              "UNKNOWN-firstname",
              "UNKNOWN-lastname",
              "UNKNOWN-email",
              "UNKNOWN-phone",
              "UNKNOWN-mobile",
              "",
              "UNKNOWN-line1",
              "UNKNOWN-line2",
              "UNKNOWN-postcode",
              "UNKNOWN-city",
              "",
              "",
              request.CountryId);


            TransactionLibrary.EditShippingInformation(
              "UNKNOWN-firstname",
              "UNKNOWN-lastname",
              "UNKNOWN-email",
              "UNKNOWN-phone",
              "UNKNOWN-mobile",
              "",
              "UNKNOWN-line1",
              "UNKNOWN-line2",
              "UNKNOWN-postcode",
              "UNKNOWN-city",
              "",
              "",
              request.CountryId);


            TransactionLibrary.CreatePayment(
                paymentMethodId: request.PaymentId,
                requestPayment: false,
                amount: -1,
                overwriteExisting: true);

            TransactionLibrary.ExecuteBasketPipeline();
            TransactionLibrary.CreateShipment(request.ShipmentId, overwriteExisting: true);
            TransactionLibrary.ExecuteBasketPipeline();

            var order = TransactionLibrary.GetBasket().PurchaseOrder;
            var discounts = order.Discounts;
            var orderTotal = Convert.ToDecimal(order.OrderTotal);

            return new SetPaymentAndShippingResponse(orderTotal, discounts);
        }
    


        public class GetShippingAndPaymentMethodsRequest
        {
            public int CountryId { get; set; }
            public string Culture { get; set; }
        }

        public class GetShippingAndPaymentMethodsResponse 
        {

            public GetShippingAndPaymentMethodsResponse()
            {
            }

            public GetShippingAndPaymentMethodsResponse(ICollection<UCommerce.EntitiesV2.ShippingMethod> shippingMethods, ICollection<UCommerce.EntitiesV2.PaymentMethod> paymentMethods, string setCulture)
            {
                ShippingMethods = new List<Shipping>();

                try {
                    Thread.CurrentThread.CurrentCulture = new CultureInfo(setCulture);
                }
                catch (Exception) { 
                    
                }
                
                var culture = System.Threading.Thread.CurrentThread.CurrentCulture.Name.ToLower();
                var currency = SiteContext.Current.CatalogContext.CurrentPriceGroup.Currency;
                var priceGroup = SiteContext.Current.CatalogContext.CurrentPriceGroup;

                foreach (var method in shippingMethods)
                {
                    var Tax = new Money(method.GetPriceForCurrency(currency).Price*priceGroup.VATRate, currency);
                    var price = new Money(method.GetPriceForCurrency(currency).Price+Tax.Value, currency).ToString();
                    
                    ShippingMethods.Add(new Shipping
                    {
                        Id = method.Id,
                        Name = method.Name,
                        Description = method.GetDescription(culture).Description,
                        Price = price
                    });

                }

                PaymentMethods = new List<Uccelerate.Services.Model.Payment>();
                var umbracoHelper = new UmbracoHelper(UmbracoContext.Current);

                foreach (var method in paymentMethods)
                {
                    var price = new Money(method.GetFeeForCurrency(currency).Fee, currency).ToString();
                    var image = umbracoHelper.TypedMedia(method.ImageMediaId);
                    var imageUrl = (image  != null) ? image.Url : "";

                    PaymentMethods.Add(new Uccelerate.Services.Model.Payment
                    {
                        Id = method.Id,
                        Name = method.Name,
                        Description = method.GetDescription(culture).Description,
                        Price = price,
                        ImageUrl = imageUrl
                    });

                }
            
            }

            public ICollection<Shipping> ShippingMethods { get; set; }
            public ICollection<Uccelerate.Services.Model.Payment> PaymentMethods { get; set; }
        }

        [HttpPost]
        public GetShippingAndPaymentMethodsResponse GetShippingAndPaymentMethods(GetShippingAndPaymentMethodsRequest request)
        {      

            var shippingInformation = TransactionLibrary.GetShippingInformation();


            if (string.IsNullOrEmpty(shippingInformation.AddressName))
            {

                TransactionLibrary.EditShippingInformation(
                    "UNKNOWN-firstname",
                    "UNKNOWN-lastname",
                    "UNKNOWN-email",
                    "UNKNOWN-phone",
                    "UNKNOWN-mobile",
                    "",
                    "UNKNOWN-line1",
                    "UNKNOWN-line2",
                    "UNKNOWN-postcode",
                    "UNKNOWN-city",
                    "",
                    "",
                    request.CountryId);

                TransactionLibrary.EditBillingInformation(
                    "UNKNOWN-firstname",
                    "UNKNOWN-lastname",
                    "UNKNOWN-email",
                    "UNKNOWN-phone",
                    "UNKNOWN-mobile",
                    "",
                    "UNKNOWN-line1",
                    "UNKNOWN-line2",
                    "UNKNOWN-postcode",
                    "UNKNOWN-city",
                    "",
                    "",
                    request.CountryId);
            }

            shippingInformation = TransactionLibrary.GetShippingInformation();
            Country country = shippingInformation.Country;
            var methodList = TransactionLibrary.GetShippingMethods(country);
            var paymentMethods = TransactionLibrary.GetPaymentMethods();
            
            return new GetShippingAndPaymentMethodsResponse(methodList, paymentMethods, request.Culture);
        }       
    }
}