﻿using System;
using System.Web;
using System.Web.Http;
using UCommerce.Api;
using UCommerce.EntitiesV2;
using UCommerce.Pipelines;
using UCommerce.Runtime;
using Umbraco.Web.WebApi;

namespace UccelerateApi
{
    public class ProductReviewRequest
    {
        public string Name { get; set; }
        public string Email { get; set; }
        public string Title { get; set; }
        public string Comments { get; set; }
        public string Sku { get; set; }
        public int Rating { get; set; }
    }

    public class ProductReviewResponse 
    {

    }

    public class ProductReviewController : UmbracoApiController
    {
        [HttpPost]
        public ProductReviewResponse Add(ProductReviewRequest request)
        {
            var basket = SiteContext.Current.OrderContext.GetBasket();
            var name = request.Name;
            var email = request.Email;
            var rating = Convert.ToInt32(request.Rating) * 20;
            var reviewHeadline = request.Title;
            var reviewText = request.Comments;
            var product = CatalogLibrary.GetProduct(request.Sku);

            if (basket.PurchaseOrder.Customer == null)
            {
                basket.PurchaseOrder.Customer = new Customer() { FirstName = name, LastName = String.Empty, EmailAddress = email };
            }
            else
            {
                basket.PurchaseOrder.Customer.FirstName = name;
                if (basket.PurchaseOrder.Customer.LastName == null)
                {
                    basket.PurchaseOrder.Customer.LastName = String.Empty;
                }
                basket.PurchaseOrder.Customer.EmailAddress = email;
            }

            basket.PurchaseOrder.Customer.Save();

            string IP = HttpContext.Current.Request.UserHostAddress;

            var review = new UCommerce.EntitiesV2.ProductReview()
            {
                ProductCatalogGroup = SiteContext.Current.CatalogContext.CurrentCatalogGroup,
                ProductReviewStatus = ProductReviewStatus.SingleOrDefault(s => s.Name == "New"),
                CreatedOn = DateTime.Now,
                CreatedBy = "System",
                Product = product,
                Customer = basket.PurchaseOrder.Customer,
                Rating = rating,
                ReviewHeadline = reviewHeadline,
                ReviewText = reviewText,
                Ip = IP
            };

            product.AddProductReview(review);

            PipelineFactory.Create<ProductReview>("ProductReview").Execute(review);

            return new ProductReviewResponse();
        }
    }
}