﻿using System;
using System.Collections.Generic;
using System.Linq;
using UCommerce.Api;
using UCommerce.EntitiesV2;
using UCommerce.Runtime;
using Uccelerate.Helpers;
using Uccelerate.Services.Model;
using System.Web.Script.Serialization;
using UCommerce.Search.Facets;
using System.Globalization;
using System.Threading;
using System.Web.Http;
using Umbraco.Web.WebApi;
using Umbraco.Web;

namespace UccelerateApi
{
    public class Products
    {
        public List<Product> ProductList { get; set; }
    }
    
    public class ProductResponse {
        public ICollection<ProductVariation> Products { get; set; }
        public int TotalResults { get; set; }
        public Dictionary<string, List<FilterItem>> Facets { get; set; }
    }

    public class ProductsController : UmbracoApiController
    {
        [HttpGet]
        public ProductResponse GetProducts(int CategoryId, string Filter, int Amount, string Culture, string BaseUrl, int CurrentPage, string SortBy, int SiteId)
        {  
            // set the correct culture info for price formatting and product information
            CultureInfo ci = new CultureInfo(Culture);
            Thread.CurrentThread.CurrentCulture = ci;
            Thread.CurrentThread.CurrentUICulture = ci;
            var site = Umbraco.TypedContent(SiteId);

            var response = new ProductResponse();
            response.Facets = new Dictionary<string, List<FilterItem>>();
            var productList = new List<Product>();
            var productOutofStockList = new List<Product>();
            var filter = new JavaScriptSerializer().Deserialize<Dictionary<string, List<string>>>(Filter);
            var products = new List<UCommerce.Documents.Product>();
            List<Facet> facetsHits = new List<Facet>();
 
            var currentCategory = Category.FirstOrDefault(c => c.CategoryId == CategoryId);
            var facets = new List<Facet>();

            if (filter.Any()) { 
                foreach (var filterKey in filter.Keys)
                {

                    var currentFilter = filter[filterKey];
                    response.Facets.Add(filterKey, new List<FilterItem>());

                    if (currentFilter.Any()) {
                        var filterList = string.Join(",", currentFilter).Split(',');

                        var facet = new Facet();
                        facet.Name = filterKey;
                        facet.FacetValues = new List<FacetValue>();

                        foreach (var value in currentFilter)
                        {
                            if(value != null)
                                facet.FacetValues.Add(new FacetValue() { Value = value });
                        }

                        facets.Add(facet);
                    }
                }
            }

            products = SearchLibrary.GetProductsFor(currentCategory, facets).Where(x => x.DisplayOnSite == true).ToList();
            facetsHits = SearchLibrary.GetFacetsFor(currentCategory, facets).ToList();
            products = products.SortOn(SortBy); //sorting
 
            //build filter list
            foreach (var fc in facetsHits)
            {
                if (!response.Facets.ContainsKey(fc.Name))
                    response.Facets.Add(fc.Name, new List<FilterItem>());

                foreach (var fi in fc.FacetValues)
                {

                    if (fi.Value.Contains(',') || fi.Value.Contains('|'))
                    {
                        foreach (var value in fi.Value.Split(new[] { '|', ',' }, StringSplitOptions.RemoveEmptyEntries))
                        {
                            if (response.Facets[fc.Name].Exists(x => x.FilterName.ToLower() == value.ToLower()))
                                response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == value.ToLower()).Count = response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == value.ToLower()).Count + 1;
                            else
                                response.Facets[fc.Name].Add(new FilterItem() { DisplayName = value, FilterName = value, Count = fi.Hits });
                        }
                    }
                    else
                    {
                        if (response.Facets[fc.Name].Exists(x => x.FilterName.ToLower() == fi.Value.ToLower()))
                            response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == fi.Value.ToLower()).Count = response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == fi.Value.ToLower()).Count + 1;
                        else
                            response.Facets[fc.Name].Add(new FilterItem() { DisplayName = fi.Value, FilterName = fi.Value, Count = fi.Hits});
                    }
                }
            }

            response.TotalResults = products.Count();

            if(Amount > 0){
                products = products.Skip(CurrentPage * Amount).Take(Amount).ToList(); 
            }

            response.Products = new List<ProductVariation>();
            var catalog = SiteContext.Current.CatalogContext.CurrentCatalog;
            

            foreach (var productDocument in products)
            {
                var product = CatalogLibrary.GetProduct(productDocument.Id);
                var productModel = Uccelerate.Public.Helpers.BuildProductModel(product, Culture, BaseUrl, false, site);

                if(productModel != null)
                    response.Products.Add(productModel);            
            }

            return response;
        }
    }
}