﻿namespace UccelerateApi
{
    using System.Collections.Generic;
    using UCommerce.Api;
    using System.Linq;
    using System;
    using Uccelerate.Services.Model;
    using Umbraco.Web.WebApi;
    using System.Web.Http;
    using UCommerce.EntitiesV2;
    using System.Globalization;
    using System.Threading;
    using Uccelerate.Helpers;
    using System.Web.Script.Serialization;
    using UCommerce.Search.Facets;
    using UCommerce.Search;

    public class SearchRequest
    {
        public string Keyword { get; set; }
        public int CurrentPage { get; set; }
        public int Amount { get; set; }
        public string BaseUrl { get; set; }
        public string Shop { get; set; }
        public string Culture { get; set; }
        public string SortBy { get; set; }
        public string Filter { get; set; }
        public int SiteId { get; set; }
    }

    public class SearchResponse {
        public ICollection<ProductVariation> Products { get; set; }
        public int TotalResult { get; set; }
        public Dictionary<string, List<FilterItem>> Facets { get; set; }
    }

    public class SearchController : UmbracoApiController
    {
        [HttpPost]
        public SearchResponse Products(SearchRequest request)
        {
            var products = new List<Product>();
            var response = new SearchResponse();
            var shop = new ProductCatalogGroup();

            //culture info
            CultureInfo ci = new CultureInfo(request.Culture);
            Thread.CurrentThread.CurrentCulture = ci;
            Thread.CurrentThread.CurrentUICulture = ci;
            var site = Umbraco.TypedContent(request.SiteId);

            //to get this library install from https://apps.ucommerce.net/#!/apps/Raven-Goggles

                try {
               
                    shop = ProductCatalogGroup.Get(Int32.Parse(request.Shop)); 
                }
                catch
                {
                }

                try { 

                var searchResults = SearchHelper.ProductSearch(request.Keyword);
                var filter = new JavaScriptSerializer().Deserialize<Dictionary<string, List<string>>>(request.Filter);
                var facets = new List<Facet>();
                response.Facets = new Dictionary<string, List<FilterItem>>();

                if (filter.Any())
                {
                    foreach (var filterKey in filter.Keys)
                    {
                        var currentFilter = filter[filterKey];
                        response.Facets.Add(filterKey, new List<FilterItem>());

                        if (currentFilter.Any())
                        {
                            var filterList = string.Join(",", currentFilter).Split(',');

                            var facet = new Facet();
                            facet.Name = filterKey;
                            facet.FacetValues = new List<FacetValue>();

                            foreach (var value in currentFilter)
                            {
                                if (value != null)
                                    facet.FacetValues.Add(new FacetValue() { Value = value });
                            }

                            facets.Add(facet);
                        }
                    }
                }

                int[] searchedIds = searchResults.Select(x => x.Id).ToArray();  
                var query = SearchLibrary.FacetedQuery();
                query.WithFacets(facets);
                var facetsHits = query.ToFacets().ToList();
            
                //Getting products for the query.
                var filterResults = query.ToList();
                searchResults = searchResults.Where(x => filterResults.Any(f => f.Id == x.Id) && (shop == null || x.ExistInShop(shop))).ToList();

                if (facetsHits.Any()) { 

                    //build filter list
                    foreach (var fc in facetsHits)
                    {
                        if (!response.Facets.ContainsKey(fc.Name))
                            response.Facets.Add(fc.Name, new List<FilterItem>());

                        foreach (var fi in fc.FacetValues)
                        {

                            if (fi.Value.Contains(',') || fi.Value.Contains('|'))
                            {
                                foreach (var value in fi.Value.Split(new[] { '|', ',' }, StringSplitOptions.RemoveEmptyEntries))
                                {
                                    if (response.Facets[fc.Name].Exists(x => x.FilterName.ToLower() == value.ToLower()))
                                        response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == value.ToLower()).Count = response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == value.ToLower()).Count + 1;
                                    else
                                        response.Facets[fc.Name].Add(new FilterItem() { DisplayName = value, FilterName = value, Count = fi.Hits });
                                }
                            }
                            else
                            {
                                if (response.Facets[fc.Name].Exists(x => x.FilterName.ToLower() == fi.Value.ToLower()))
                                    response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == fi.Value.ToLower()).Count = response.Facets[fc.Name].FirstOrDefault(x => x.FilterName.ToLower() == fi.Value.ToLower()).Count + 1;
                                else
                                    response.Facets[fc.Name].Add(new FilterItem() { DisplayName = fi.Value, FilterName = fi.Value, Count = fi.Hits });
                            }
                        }

                    }

                }

                response.Products = new List<ProductVariation>();
                searchResults = searchResults.ToList().SortOn(request.SortBy); //sorting
                response.TotalResult = searchResults.Count();

                if (request.Amount > 0)
                {
                    searchResults = searchResults.Skip(request.CurrentPage * request.Amount).Take(request.Amount).ToList();
                }

                foreach (var productDocument in searchResults)
                {
                    var product = CatalogLibrary.GetProduct(productDocument.Id);
                    var productModel = Uccelerate.Public.Helpers.BuildProductModel(product, request.Culture, request.BaseUrl, false, site);

                    if (productModel != null)
                        response.Products.Add(productModel);
                }             
            
                } catch( Exception ex) { }
            
            return response;
        }
    }
}