Vue.component('category', {
  props: ['culture', 'category', 'productsPerPage', 'defaultSort', 'siteId'],
  data: function () {
    return {
      currentPage: 0,
      products: [],
      sortSelect: "name",
      filter: {},
      filterList: [],
      showLoader: false,
      filterCount: 0,
      allLoaded: false
    }
  },
  mounted: function () {
    this.sortSelect = this.defaultSort;
    this.LoadProducts(false);
    
    Helpers.customSelect();
  },
  methods: {
    LoadProducts: function (append) {
      this.InitLoader();
        
      this.filterCount = 0;
      
      for (var prop in this.filter) {
        this.filterCount += this.filter[prop].length;
      }

      var obj = this;
      axios.get("/Umbraco/Api/Products/GetProducts", {
        params: {
          BaseUrl: window.location.href,
          Filter: JSON.stringify(this.filter),
          Amount: this.productsPerPage,
          CurrentPage: this.currentPage,
          CategoryId: this.category,
          SortBy: this.sortSelect,
          Culture: this.culture,
          SiteId: this.siteId
        }
      }).then(function (response) {
        obj.ScrapLoader();
        obj.filterList = response.data.Facets;
        
        if (!append)
          obj.products = response.data.Products;
        else
          obj.products = obj.products.concat(response.data.Products);

        for (var key in obj.filterList) {
          if (!(key in obj.filter))
            obj.filter[key] = [];
        }

        if (response.data.TotalResults == obj.products.length)
          obj.allLoaded = true;
      });
    },
    OpenFilter: function(){
        this.$root.$emit('openFilter');
    },
    CloseFilter: function(){
        this.$root.$emit('closeFilter');
    },
    ClearFilter: function () {
      this.filterCount = 0;
      this.filter = {
      };
    },
    ChangeFilter: function () {
      this.products = [];
      this.currentPage = 0;
      this.LoadProducts(false);    
    },
    ToggleFilterPanel: function(event){
        var parent = event.currentTarget.parentElement;
        parent.classList.toggle("store-filter-section--open");
    },
    LoadMore: function () {
      this.currentPage++;
      this.LoadProducts(true);
    },
    ChangeSort: function () {
      this.products = [];
      this.currentPage = 0;
      this.LoadProducts(false);
    },
    InitLoader: function () {
      var placeholders = [];

      for (var i = 0; i < this.productsPerPage; i++) {
        var p = { ProductName: i };
        p.ProductName = i;
        placeholders.push(p);
      }
      
      this.products = this.products.concat(placeholders);
    },
    ScrapLoader: function () {
      this.products.splice(this.products.length - this.productsPerPage, this.productsPerPage);
    }
  }
});