Vue.component('product', {
  props: ['culture', 'productId', 'variants', 'siteId'],
  data: function () {
    return {
      showMessage: false,
      review: {
          Rating: 0
      },
      foundMatch: false,
      pickerCount: 0,
      product: {},
      pickedVariant: "",
      variantPicker: [],
      variantsJson: [],
      activeTab: "description"
    }
  },
  mounted: function () {
      if(this.variants)
        this.variantsJson = JSON.parse(this.variants);
        
        //flickity re-init after lazysizes hav loaded
        document.addEventListener("lazyloaded", function(){
            Flickity.data(document.querySelector(".product-post__media")).resize(); 
        });
        
      this.loadProduct();
 
        //set default variant
        for (var key in this.variantsJson[0]) {
            if(key != "Sku"){
                this.variantPicker[key] = this.variantsJson[0][key];
            }
        }
        this.changeVariant();
        
         var flkty = new Flickity( '.product-post__media', {
            cellAlign: 'left',
    	    contain: 1,
    	    prevNextButtons: 1,
    	    pageDots: 1,
    	    setGallerySize: 1,
    	    imagesLoaded: 1,
    	    wrapAround: 0,
    	    autoPlay: 0
        });
        
  },
  methods: {
    loadProduct: function(){
        var obj = this;
        axios.post("/Umbraco/Api/Product/GetProduct", { ProductId: this.productId, Culture: this.culture, SiteId: this.siteId }).then(function (response) {
            obj.product = response.data.Product;
            
            //fix to make sure flickity always loads
            setTimeout(function(){
                Flickity.data(document.querySelector(".product-post__media")).resize(); 
            }, 500)
        });       
    },
    submitReview: function (e) {
        e.preventDefault();
        var obj = this;
          axios.post("/Umbraco/Api/ProductReview/Add", { Title: this.review.Title, Comments: this.review.Comments, Name: this.review.Name, Email: this.review.Email, Rating: this.review.Rating, Sku: this.product.Sku }).then(function (response) {
            obj.showMessage = true;
          });
    },
    changeVariant: function () {
      this.foundMatch = false;
      this.pickerCount = 0;

      for (var i = 0; i < this.variantsJson.length; i++) {
        var matches = 0;
    
        for (var key in this.variantPicker) {
          if (this.variantPicker[key].replace(/ /g, '').toLowerCase() == this.variantsJson[i][key].replace(/ /g, '').toLowerCase()) {
            matches++;
          }
        }

        if (matches === Object.keys(this.variantPicker).length) {
          this.pickedVariant = this.variantsJson[i].Sku;
          this.foundMatch = true;
        }
        
        this.pickerCount = Object.keys(this.variantPicker).length;
      }
    },
    galleryItemClick: function(e){
        e.preventDefault();
        
        var getItems = function(gallery) {
            var items = [],
            links = gallery.querySelectorAll('a.product-post__toggle-gallery');

            for(var j = 0; j < links.length; j++) {
                var href   = links[j].getAttribute('href'),
                    size   = links[j].getAttribute('data-size').split('x'),
                    width  = size[0],
                    height = size[1];

                var item = {
                    src : href,
                    w   : width,
                    h   : height
                };
 
                items.push(item);
            }
            return items;
        }
         var index = Helpers.indexInParent(e.target.parentNode.parentNode);
         var pswp = document.getElementsByClassName("pswp")[0];
         var items = getItems(e.target.parentNode.parentNode.parentNode);

          var options = {
              index: index,
              bgOpacity: 0.9,
              showHideOpacity: true,
              shareEl: false,
              getThumbBoundsFn: function(index) {
                var thumbnail = document.querySelectorAll('.pw-thumb')[index];
                var pageYScroll = window.pageYOffset || document.documentElement.scrollTop; 
                var rect = thumbnail.getBoundingClientRect(); 
                return {x:rect.left, y:rect.top + pageYScroll, w:rect.width};
            }
          }
           
          // Initialize PhotoSwipe
          var lightBox = new PhotoSwipe(pswp, PhotoSwipeUI_Default, items, options);
          lightBox.init();       
    },
    imageLoaded: function(){
        Flickity.data(document.querySelector(".product-post__media")).resize(); 
    }
  }
});