Vue.component('search', {
  props: ['culture', 'productsPerPage', 'defaultSort', 'initialQuery', 'siteId', 'searchShop'],
  data: function () {
    return {
      query: "",
      currentPage: 0,
      products: [],
      sortSelect: "name",
      filter: {},
      filterList: [],
      showLoader: false,
      filterCount: 0,
      allLoaded: true,
      totalResults: 0,
      searchLoading: false
    }
  },
  mounted: function () {
    this.query = this.initialQuery;
    this.sortSelect = this.defaultSort;
    Helpers.customSelect();

    if (this.query != null)
      this.LoadProducts(false);
  },
  methods: {
    LoadProducts: function (append, hideLoader) {

      if (!hideLoader)
        this.InitLoader();

      this.filterCount = 0;

      for (var prop in this.filter) {
        this.filterCount += this.filter[prop].length;
      }

      this.searchLoading = true;

      var obj = this;
      axios.post("/Umbraco/Api/search/Products", {
        KeyWord: obj.query,
        BaseUrl: "",
        Filter: JSON.stringify(this.filter),
        Amount: this.productsPerPage,
        CurrentPage: this.currentPage,
        CategoryList: this.categories,
        SortBy: this.sortSelect,
        Culture: this.culture,
        SiteId: this.siteId,
        Shop: this.searchShop
      }).then(function (response) {

        if (!hideLoader)
          obj.ScrapLoader();

        obj.searchLoading = false;
        obj.totalResults = response.data.TotalResult;
        obj.filterList = response.data.Facets;

        if (response.data.Products != null) {
          if (!append) {
            obj.products = response.data.Products;
          } else {
            obj.products = obj.products.concat(response.data.Products);
          }
        }

        for (var key in obj.filterList) {
          if (!(key in obj.filter))
            obj.filter[key] = [];
        }

        if (response.data.TotalResult == obj.products.length)
          obj.allLoaded = true;
        else
          obj.allLoaded = false;

      });

    },
    postSearch: function (miniSearch) {
      if (this.query.length > 0)
        this.LoadProducts(false, miniSearch);

      if (!miniSearch)
        window.history.pushState({ path: window.location.pathname + "?search=" + this.query }, '', window.location.pathname + "?search=" + this.query); //save query to url
    },
    OpenFilter: function () {
      this.$root.$emit('openFilter');
    },
    CloseFilter: function () {
      this.$root.$emit('closeFilter');
    },
    ClearFilter: function () {
      this.filterCount = 0;
      this.filter = {
      };
    },
    ChangeFilter: function () {
      this.products = [];
      this.currentPage = 0;
      this.LoadProducts(false);
    },
    ToggleFilterPanel: function (event) {
      var parent = event.currentTarget.parentElement;
      parent.classList.toggle("store-filter-section--open");
    },
    LoadMore: function () {
      this.currentPage++;
      this.LoadProducts(true);
    },
    ChangeSort: function () {
      this.products = [];
      this.currentPage = 0;
      this.LoadProducts(false);
    },
    InitLoader: function () {
      var placeholders = [];

      for (var i = 0; i < this.productsPerPage; i++) {
        var p = { ProductName: i };
        p.ProductName = i;
        placeholders.push(p);
      }

      this.products = this.products.concat(placeholders);
    },
    ScrapLoader: function () {
      this.products.splice(this.products.length - this.productsPerPage, this.productsPerPage);
    }
  }
});